/*
--------------------------------------------------------------------------------------
Copyright 1992 Edwin D. Barnes CIS:74730,1111
--------------------------------------------------------------------------------------
References:
     Microsoft SDK manuals
     Borland C++ & Object Windows manuals
     Turbo Pascal for Windows Programming by Tom Swan
      (How to access system menu, Page 268)

--------------------------------------------------------------------------------------
*** Revision History ***
--------------------------------------------------------------------------------------
created May 11 1992

revised May 15 1992
     added icons to show ON/OFF state
     removed menu and put choices into system menu
     updated about box
     added always on top
     added animated icon when activated and red icon when stoped
     updated revision to 1.1

revised May 26 1992
     added menu toggle for always on top
     updated revision to 1.2
     released to compuserve BCPPWIN forum 

revised May 27 1992
     fixed "stopped" icon update every second
     updated to revision 1.3

revised June 2 1992
     fixed the iconic problem where app wouldn't stay iconic.
     removed minimize from the system menu.
     changed offsets for toggles in system menus.
     updated to revision 1.4

--------------------------------------------------------------------------------------
*** WARNINGS ***
--------------------------------------------------------------------------------------
Warning this application will SLOW windows. Use it for diagnostic purposes only!!!!!
I didn't want to use a timer to regulate update rate.

--------------------------------------------------------------------------------------
*** NOTES ***
--------------------------------------------------------------------------------------

0) If you discover any memory or resource leaks please notify me on compuserve.
   Any improvements or suggestions would be appreciated too.

*/


#include <owl.h>
#include "TIMECNT.H"

_CLASSDEF (PTTimerApp)

class TTimerApp : public TApplication
{

public:

  TTimerApp(LPSTR, HINSTANCE , HINSTANCE ,LPSTR , int );
//    : TApplication(AName, hInstance, hPrevInstance, lpCmdLine, nCmdShow);

  ~TTimerApp();
  virtual void InitMainWindow();
  virtual void IdleAction();
};


_CLASSDEF(TTimerWin)
class TTimerWin : public TWindow
{
public:
  int ON;	 // stop (ON/OFF) flag
  int OldCount;  // previous timers free count
  int IconCount; // icon animation frame counter
  int ONTOP;     // always on top status flag
  int Beep;      // beep on/off status flag

  DWORD TickCount;  // count for one second delay
  HICON IG[9],IR;   // icon handels


  TTimerWin(PTWindowsObject AParent, LPSTR ATitle);
  ~TTimerWin();

  virtual void WMSysCommand(RTMessage Msg) = [WM_FIRST+WM_SYSCOMMAND];
  virtual void GetWindowClass(WNDCLASS& WndClass );
  virtual void SetupWindow();
  virtual void Paint(HDC DC, PAINTSTRUCT _FAR & PaintStruct);
  virtual void WMQueryOpen(RTMessage) = [WM_FIRST + WM_QUERYOPEN];
	  void CMAbout();
	  void CMTglStop();
	  void CMTglTop();
          void CMTglBeep();
	  void CMTimerCheck(); // IdleAction calls this to check Timers
};



TTimerWin::TTimerWin(PTWindowsObject AParent, LPSTR ATitle)
  : TWindow(AParent, ATitle)
{

  ON=TRUE;	     // default app running
  ONTOP=TRUE;	     // default always on top	
  Beep=TRUE;         // default beep on
  OldCount=0;        // previous number of timers used
  IconCount=0;       // default frame counter at first frame
  TickCount=GetTickCount(); // get current time for one second delay

  Attr.W=130;
  Attr.H=60;
  Attr.Style= WS_OVERLAPPED | WS_SYSMENU | WS_MINIMIZEBOX;

}

TTimerWin::~TTimerWin()
{
 // cleanup code here
}

void TTimerWin::GetWindowClass(WNDCLASS& WndClass)
{
  TWindow::GetWindowClass(WndClass);
  WndClass.hIcon=NULL;
  WndClass.hbrBackground=(HBRUSH)GetStockObject(BLACK_BRUSH);
}


void TTimerWin::SetupWindow()
{
 TWindow::SetupWindow();
 HMENU hMenu=GetSystemMenu(HWindow,FALSE);
 DeleteMenu(hMenu,SC_MAXIMIZE,MF_BYCOMMAND);
 DeleteMenu(hMenu,SC_MINIMIZE,MF_BYCOMMAND);
 DeleteMenu(hMenu,SC_SIZE    ,MF_BYCOMMAND);
 DeleteMenu(hMenu,SC_RESTORE ,MF_BYCOMMAND);
 AppendMenu(hMenu,MF_SEPARATOR,NULL,NULL);
 AppendMenu(hMenu,MF_STRING,CM_FIRST+CM_ABOUT,"About...");
 AppendMenu(hMenu,MF_STRING|MF_BYPOSITION|MF_UNCHECKED,CM_FIRST+CM_STOP,"Stop");
 AppendMenu(hMenu,MF_STRING|MF_BYPOSITION|MF_CHECKED,CM_FIRST+CM_TglTop,"Always on top");
 AppendMenu(hMenu,MF_STRING|MF_BYPOSITION|MF_CHECKED,CM_FIRST+CM_BEEP,  "Beep");

 // load icons
 for(int i=0;i<9;i++)
  {
   char tmp[20];
   wsprintf(tmp,"IG_%d",i);
   IG[i]=LoadIcon(GetApplication()->hInstance,tmp);
  }

 IR=LoadIcon(GetApplication()->hInstance,"IR");

}

void TTimerWin::Paint(HDC DC, PAINTSTRUCT _FAR & PaintInfo)
{
   if(ON)
     {
       DrawIcon(DC,2,2,IG[IconCount]);
       if (++IconCount >8) IconCount=0;
     }
    else
     DrawIcon(DC,2,2,IR);
}

// This is what keeps this program minimized
void TTimerWin::WMQueryOpen(RTMessage Msg)
{
 Msg.Result=0;
}

// decode system menu messages
void TTimerWin::WMSysCommand(RTMessage Msg)
{

  switch (Msg.WParam)
  {
   case CM_FIRST+CM_ABOUT:
		  CMAbout();
		  break;

   case CM_FIRST+CM_STOP:
		  CMTglStop();
		  break;

   case CM_FIRST+CM_BEEP:
		  CMTglBeep();
		  break;

   case CM_FIRST+CM_TglTop:
		  CMTglTop();
		  break;

   default:
		  DefWndProc(Msg);
  }

}

void TTimerWin::CMAbout()
{
 GetApplication()->ExecDialog(new TDialog(this, "About"));
}

void TTimerWin::CMTglStop()
{
  // toggle "Stop"/"GO" state
  HMENU hMenu=GetSystemMenu(HWindow,FALSE);

  if (ON)
   ModifyMenu(hMenu,7,MF_STRING|MF_BYPOSITION|MF_CHECKED  ,CM_FIRST+CM_STOP,"Stop");
  else
   ModifyMenu(hMenu,7,MF_STRING|MF_BYPOSITION|MF_UNCHECKED,CM_FIRST+CM_STOP,"Stop");

  DrawMenuBar(HWindow);

  // toggle update flag
  ON=!ON;

  // force a WM_PAINT so the appropiate icon will be drawn. 
  InvalidateRect(HWindow,NULL,TRUE);

}

void TTimerWin::CMTglTop()
{
  // toggle "Always on top" state
  HMENU hMenu=GetSystemMenu(HWindow,FALSE);

  if (ONTOP) 
   ModifyMenu(hMenu,8,MF_STRING|MF_BYPOSITION|MF_UNCHECKED,
              CM_FIRST+CM_TglTop,"Always on top");

  else
   ModifyMenu(hMenu,8,MF_STRING|MF_BYPOSITION|MF_CHECKED,
	      CM_FIRST+CM_TglTop,"Always on top");

  DrawMenuBar(HWindow);

  // toggle flag
  ONTOP=!ONTOP;
}

void TTimerWin::CMTglBeep()
{
  // toggle "BEEP" state
  HMENU hMenu=GetSystemMenu(HWindow,FALSE);

  if (Beep) 
   ModifyMenu(hMenu,9,MF_STRING|MF_BYPOSITION|MF_UNCHECKED,
              CM_FIRST+CM_BEEP,"Beep");

  else
   ModifyMenu(hMenu,9,MF_STRING|MF_BYPOSITION|MF_CHECKED,
	      CM_FIRST+CM_BEEP,"Beep");

  DrawMenuBar(HWindow);

  // toggle flag
  Beep=!Beep;
}


void TTimerWin::CMTimerCheck()
{

 // update anamated icon ... not cool to use a timer in this app  so use tick count...
 if((GetTickCount()-TickCount) > 1000)  // test if a second has gone by. 
 {
  TickCount=GetTickCount();	// always use current count to prevent flashing
				// numerous icons if app is suspended for awhile
  if(ONTOP)
  SetWindowPos(HWindow,
	       NULL,
	       0,0,0,0,
	       SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);

  if (ON) 
   InvalidateRect(HWindow,NULL,TRUE); // force a PAINT message so a new icon will be drawn
 }

 if(!ON) return; //  return if stopped

 int i=0; // timer counter

 // capture & count all available timers
 while (SetTimer(HWindow,i,10000,NULL)) i++;

 // release captured timers
 if(i)
   for(int j=0;j<i;j++) KillTimer(HWindow,j);


 if (i==OldCount) return;  // don't update caption if no change in state
  else
  {
    // update caption with current timer count
    char MyCaption[20];

    wsprintf(MyCaption,"Timers:%d",i);

    SetCaption((LPSTR)MyCaption);
    OldCount=i; // save current count
    if(Beep) MessageBeep(0);
  }
}


// force appp to start minimized 
TTimerApp::TTimerApp(LPSTR lpzNam,HINSTANCE hInst,HINSTANCE hPrevInst,LPSTR lpzCmdLn,int nCmdShw)
: TApplication(lpzNam,hInst,hPrevInst,lpzCmdLn,nCmdShw)
{
 nCmdShow=SW_SHOWMINIMIZED;
}

TTimerApp::~TTimerApp()
{
}

void TTimerApp::IdleAction()
{
  // NOTE!!!!!!!!!!!!!!
  // When you use IdleAction don't do long calculations otherwise you will
  // slow down the responsiveness of windows. Break your calculations into
  // small quick pieces if possible.

  PTTimerWin(MainWindow)->CMTimerCheck();
}


void TTimerApp::InitMainWindow()
{
  MainWindow = new TTimerWin(NULL, Name);
}


int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpCmdLine, int nCmdShow)
{
  if (hPrevInstance == NULL)
  {
   TTimerApp TimerApp("TimerCounter", hInstance, hPrevInstance,lpCmdLine,nCmdShow);
   TimerApp.Run();
   return TimerApp.Status;
  }
  else return NULL;
}
